"use server";

import { requestGetFetch, requestPostFetch } from "@/utils/fetch";
import { getServerSession, Session } from "next-auth";
import { nextauthOptions } from "../nextauth-options";
import { SearchParams } from "@/types/searchParams";

export async function getConsumers(searchParams?: SearchParams) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;

    // Dynamically build the query string from searchParams
    const paramsArray = searchParams && Object.entries(searchParams);
    const queryString = paramsArray
      ? paramsArray
          .map(([key, value]) =>
            Array.isArray(value)
              ? `${key}=${value.join(",")}` // Use .join(",") without encoding commas
              : `${key}=${String(value)}` // Encode only non-array values
          )
          .join("&")
      : "";

    const path = "v1/consumers" + (queryString ? `?${queryString}` : "");

    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });

    return response;
  } catch (error) {
    throw error;
  }
}

export async function deleteConsumer(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/consumers/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        method: "DELETE",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response;
  } catch (error) {
    throw error;
  }
}

interface CreateConsumerProps {
  name: string;
  permissions: string[];
}

export async function createConsumer({
  name,
  permissions,
}: CreateConsumerProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const response = await requestPostFetch({
      path: `v1/consumers`,
      data: {
        name,
        permissions,
      },
      token: token,
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}

interface UpdateConsumerProps {
  id: number;
  name: string;
  permissions: string[];
}

export async function updateConsumer({
  id,
  name,
  permissions,
}: UpdateConsumerProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/consumers/${id}`;

    const response = await requestGetFetch({
      path,
      options: {
        method: "PUT",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        body: JSON.stringify({
          name,
          permissions,
        }),
        cache: "no-store",
      },
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}
